@@ -63,7 +63,8 @@ enum
   IMAP_NEW_MAIL,
   IMAP_EXPUNGE,
   IMAP_BYE,
-  IMAP_OK_FAIL
+  IMAP_OK_FAIL,
+  IMAP_REOPENED
 };
 
 /* Capabilities */
@@ -135,6 +136,7 @@ typedef struct
   /* This data is specific to a CONNECTION to an IMAP server */
   short status;
   short state;
+  short check_status;
   char delim;
   unsigned char capabilities[(CAPMAX + 7)/8];
   CONNECTION *conn;
@@ -1047,10 +1049,12 @@ static int imap_exec (char *buf, size_t buflen, IMAP_DATA *idata,
 
       count = imap_read_headers (idata->selected_ctx, 
 	  idata->selected_ctx->msgcount, count - 1) + 1;
+      idata->check_status = IMAP_NEW_MAIL;
     }
     else
     {
       imap_reopen_mailbox (idata->selected_ctx, NULL);
+      idata->check_status = IMAP_REOPENED;
     }
 
     idata->status = 0;
@@ -1332,7 +1336,16 @@ static int imap_open_connection (IMAP_DATA *idata, CONNECTION *conn)
       return (-1);
     }
   }
-  else if (mutt_strncmp ("* PREAUTH", buf, 9) != 0)
+  else if (mutt_strncmp ("* PREAUTH", buf, 9) == 0)
+  {
+    if (imap_check_capabilities(idata) != 0)
+    {
+      close (conn->fd);
+      idata->state = IMAP_DISCONNECTED;
+      return (-1);
+    }
+  } 
+  else
   {
     imap_error ("imap_open_connection()", buf);
     close (conn->fd);
@@ -1625,7 +1638,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   mutt_socket_write (CTX_DATA->conn, buf);
   do
   {
-    if (mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn) < 0)
+    if (mutt_socket_read_line_d (buf, sizeof (buf), CTX_DATA->conn) < 0)
     {
       return (-1);
     }
@@ -1637,19 +1650,35 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
       pc = imap_next_word (pc);
       if (mutt_strncasecmp ("FETCH", pc, 5) == 0)
       {
-        if (get_literal_count(buf, &bytes) < 0)
+	while (*pc)
 	{
-	  imap_error ("imap_fetch_message()", buf);
-	  return (-1);
-	}
-	for (pos = 0; pos < bytes; )
-	{
-	  len = mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn);
-	  if (len < 0)
-	    return (-1);
-	  pos += len;
-	  fputs (buf, msg->fp);
-	  fputs ("\n", msg->fp);
+	  pc = imap_next_word (pc);
+	  if (pc[0] == '(')
+	    pc++;
+	  dprint (2, (debugfile, "Found FETCH word %s\n", pc));
+	  if (strncasecmp ("RFC822", pc, 6) == 0)
+	  {
+	    pc = imap_next_word (pc);
+	    if (get_literal_count(pc, &bytes) < 0)
+	    {
+	      imap_error ("imap_fetch_message()", buf);
+	      return (-1);
+	    }
+	    for (pos = 0; pos < bytes; )
+	    {
+	      len = mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn);
+	      if (len < 0)
+		return (-1);
+	      pos += len;
+	      fputs (buf, msg->fp);
+	      fputs ("\n", msg->fp);
+	    }
+	    if (mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn) < 0)
+	    {
+	      return (-1);
+	    }
+	    pc = buf;
+	  }
 	}
       }
       else if (imap_handle_untagged (CTX_DATA, buf) != 0)
@@ -1843,11 +1872,24 @@ int imap_close_connection (CONTEXT *ctx)
   return 0;
 }
 
+static void _imap_set_flag (CONTEXT *ctx, int aclbit, int flag, const char *str, 
+		     char *sf, char *uf)
+{
+  if (mutt_bit_isset (CTX_DATA->rights, aclbit))
+  {
+    if (flag)
+      strcat (sf, str);
+    else
+      strcat (uf, str);
+  }
+}
+
 int imap_sync_mailbox (CONTEXT *ctx)
 {
   char seq[8];
   char buf[LONG_STRING];
-  char tmp[LONG_STRING];
+  char set_flags[LONG_STRING];
+  char unset_flags[LONG_STRING];
   int n;
 
   /* save status changes */
@@ -1855,32 +1897,43 @@ int imap_sync_mailbox (CONTEXT *ctx)
   {
     if (ctx->hdrs[n]->deleted || ctx->hdrs[n]->changed)
     {
-      snprintf (tmp, sizeof (tmp), _("Saving message status flags... [%d/%d]"),
-	n+1, ctx->msgcount);
-      mutt_message (tmp);
-      *tmp = 0;
-      if (ctx->hdrs[n]->read && mutt_bit_isset(CTX_DATA->rights, IMAP_ACL_SEEN))
-	strcat (tmp, "\\Seen ");
-      if (mutt_bit_isset(CTX_DATA->rights, IMAP_ACL_WRITE))
+      snprintf (buf, sizeof (buf), _("Saving message status flags... [%d/%d]"),
+		n+1, ctx->msgcount);
+      mutt_message (buf);
+      
+      *set_flags = '\0';
+      *unset_flags = '\0';
+      
+      _imap_set_flag (ctx, IMAP_ACL_SEEN, ctx->hdrs[n]->read, "\\Seen ", set_flags, unset_flags);
+      _imap_set_flag (ctx, IMAP_ACL_WRITE, ctx->hdrs[n]->flagged, "\\Flagged ", set_flags, unset_flags);
+      _imap_set_flag (ctx, IMAP_ACL_WRITE, ctx->hdrs[n]->replied, "\\Answered ", set_flags, unset_flags);
+      _imap_set_flag (ctx, IMAP_ACL_DELETE, ctx->hdrs[n]->deleted, "\\Deleted", set_flags, unset_flags);
+      
+      mutt_remove_trailing_ws (set_flags);
+      mutt_remove_trailing_ws (unset_flags);
+      
+      if (*set_flags)
       {
-	if (ctx->hdrs[n]->flagged)
-	  strcat (tmp, "\\Flagged ");
-	if (ctx->hdrs[n]->replied)
-	  strcat (tmp, "\\Answered ");
+	imap_make_sequence (seq, sizeof (seq));
+	snprintf (buf, sizeof (buf), "%s STORE %d +FLAGS.SILENT (%s)\r\n", seq,
+		  ctx->hdrs[n]->index + 1, set_flags);
+	if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
+	{
+	  imap_error ("imap_sync_mailbox()", buf);
+	  return (-1);
+	}
       }
-      if (ctx->hdrs[n]->deleted && 
-	  mutt_bit_isset(CTX_DATA->rights, IMAP_ACL_DELETE))
-        strcat (tmp, "\\Deleted");
-      mutt_remove_trailing_ws (tmp);
-
-      if (!*tmp) continue; /* imapd doesn't like empty flags. */
-      imap_make_sequence (seq, sizeof (seq));
-      snprintf (buf, sizeof (buf), "%s STORE %d FLAGS.SILENT (%s)\r\n", seq, 
-      	ctx->hdrs[n]->index + 1, tmp);
-      if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
+      
+      if (*unset_flags)
       {
-	imap_error ("imap_sync_mailbox()", buf);
-	return (-1);
+	imap_make_sequence (seq, sizeof (seq));
+	snprintf (buf, sizeof (buf), "%s STORE %d -FLAGS.SILENT (%s)\r\n", seq,
+		  ctx->hdrs[n]->index + 1, unset_flags);
+	if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
+	{
+	  imap_error ("imap_sync_mailbox()", buf);
+	  return (-1);
+	}
       }
     }
   }
@@ -1959,7 +2012,14 @@ void imap_fastclose_mailbox (CONTEXT *ctx)
   }
 }
 
-/* use the NOOP command to poll for new mail */
+/* use the NOOP command to poll for new mail
+ *
+ * return values:
+ *	M_REOPENED	mailbox has been reopened
+ *	M_NEW_MAIL	new mail has arrived!
+ *	0		no change
+ *	-1		error
+ */
 int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
   char seq[8];
@@ -1974,6 +2034,7 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
     checktime=k;
   }
 
+  CTX_DATA->check_status = 0;
   imap_make_sequence (seq, sizeof (seq));
   snprintf (buf, sizeof (buf), "%s NOOP\r\n", seq);
   if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
@@ -1982,7 +2043,11 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
     return (-1);
   }
 
-  return (msgcount != ctx->msgcount);
+  if (CTX_DATA->check_status == IMAP_NEW_MAIL)
+    return M_NEW_MAIL;
+  if (CTX_DATA->check_status == IMAP_REOPENED)
+    return M_REOPENED;
+  return 0;
 }
 
 int imap_buffy_check (char *path)
