@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: menu.c,v 1.11 1998/12/10 11:06:25 roessler Exp $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  *
@@ -275,13 +274,19 @@ void menu_redraw_current (MUTTMENU *menu)
 
 void menu_check_recenter (MUTTMENU *menu)
 {
-  if (menu->current >= menu->top + menu->pagelen)
+  if (menu->max <= menu->pagelen && menu->top != 0)
+  {
+    menu->top = 0;
+    set_option (OPTNEEDREDRAW);
+    menu->redraw |= REDRAW_INDEX;
+  }
+  else if (menu->current >= menu->top + menu->pagelen)
   {
     if (option (OPTMENUSCROLL))
       menu->top = menu->current - menu->pagelen + 1;
     else
       menu->top += menu->pagelen * ((menu->current - menu->top) / menu->pagelen);
-    menu->redraw = REDRAW_INDEX;
+    menu->redraw |= REDRAW_INDEX;
   }
   else if (menu->current < menu->top)
   {
@@ -293,7 +298,7 @@ void menu_check_recenter (MUTTMENU *menu)
       if (menu->top < 0)
 	menu->top = 0;
     }
-    menu->redraw = REDRAW_INDEX;
+    menu->redraw |= REDRAW_INDEX;
   }
 }
 
